@@ -15,7 +15,7 @@
 
 mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
                       start = NULL, etastart = NULL, mustart = NULL,
-                      offset = rep(0, nobs), family = gaussian(), 
+                      offset = rep(0, nobs), family = gaussian(),
                       control = list(), intercept = TRUE)
 {
     control <- do.call("mhglm.control", control)
@@ -23,7 +23,7 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
     z <- as.matrix(z)
     xnames <- dimnames(x)[[2L]]
     znames <- dimnames(z)[[2L]]
-    ynames <- if (is.matrix(y)) 
+    ynames <- if (is.matrix(y))
         rownames(y)
     else names(y)
     nobs <- NROW(y)
@@ -35,7 +35,8 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
 
     if (!is.null(start)) {
         if (length(start) != nfixed) {
-            stop(gettextf("length of 'start' should equal %d and correspond to initial coefs for %s",
+            stop(gettextf(paste0("length of 'start' should equal %d",
+                                 " and correspond to initial coefs for %s"),
                           nfixed, paste(deparse(xnames), collapse=", ")),
                  domain=NA)
         }
@@ -43,10 +44,13 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
     }
 
     # group-specific estimates
-    m <- rdglm.group.fit(x = cbind(x, z), y = y, group = group, weights = weights,
-                         start = start, etastart = etastart, mustart = mustart,
-                         offset = offset, family = family,
-                         control = control$fit.control, method = control$fit.method,
+    m <- rdglm.group.fit(x = cbind(x, z), y = y, group = group,
+                         weights = weights, start = start,
+                         etastart = etastart, mustart = mustart,
+                         offset = offset, family = family, 
+                         parallel = control$parallel,
+                         control = control$fit.control,
+                         method = control$fit.method,
                          intercept = intercept)
     ngroups <- m$ngroups
 
@@ -60,25 +64,48 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
 
     # compute group-sqecific precision square root (unpivoted)
     Rp <- as.list(rep(NULL, ngroups))
-    for (i in seq_len(ngroups)) {
-        qr.i <- m$qr[[i]]
-        rank.i <- qr.i$rank
-        Rp[[i]] <- matrix(0, rank.i, nvars)
-        if (rank.i > 0L) {
-            R.i <- qr.R(qr.i)
-            pivot.i <- qr.i$pivot
-            Rp[[i]][,pivot.i] <- R.i[seq_len(rank.i),]
+
+    if(control$parallel) {
+        Rp <- foreach(i = seq_len(ngroups)) %dopar% {
+            qr.i <- m$qr[[i]]
+            rank.i <- qr.i$rank
+            Rp <- matrix(0, rank.i, nvars)
+            if (rank.i > 0L) {
+                R.i <- qr.R(qr.i)
+                pivot.i <- qr.i$pivot
+                Rp[, pivot.i] <- R.i[seq_len(rank.i), ]
+            }
+            return(Rp)
+        }
+    } else {
+        for(i in seq_len(ngroups)) {
+            qr.i <- m$qr[[i]]
+            rank.i <- qr.i$rank
+            Rp[[i]] <- matrix(0, rank.i, nvars)
+            if (rank.i > 0L) {
+                R.i <- qr.R(qr.i)
+                pivot.i <- qr.i$pivot
+                Rp[[i]][,pivot.i] <- R.i[seq_len(rank.i),]
+            }
         }
     }
 
     # change coordinates so that average precision is identity
+
     if (control$standardize) {
         # compute averge precision of estimates
         prec.avg <- matrix(0, nvars, nvars)
-
-        for (i in seq_len(ngroups)) {
-            scale.i <- sqrt(1/dispersion[i])
-            prec.avg <- prec.avg + crossprod(scale.i * Rp[[i]])
+        if(control$parallel) {
+            prec.avg <- foreach(i = seq_len(ngroups)) %dopar% {
+                scale.i <- sqrt(1/dispersion[i])
+                return(crossprod(scale.i * Rp[[i]]))
+            }
+            prec.avg <- Reduce('+', prec.avg)
+        } else {
+            for (i in seq_len(ngroups)) {
+                scale.i <- sqrt(1/dispersion[i])
+                prec.avg <- prec.avg + crossprod(scale.i * Rp[[i]])
+            }
         }
         prec.avg <- prec.avg / ngroups
 
@@ -96,7 +123,8 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
         #r1.random <- seq_len(attr(R.random, "rank"))
         #R.fixed <- R.fixed[r1.fixed,r1.fixed,drop=FALSE]
         #R.random <- R.random[r1.random,r1.random,drop=FALSE]
-    } else {
+    }
+    else { # control.standardize==FALSE
         R.fixed <- diag(nfixed)
         attr(R.fixed, "pivot") <- seq_len(nfixed)
         attr(R.fixed, "rank") <- nfixed
@@ -132,32 +160,53 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
     coef <- m$coefficients[,pivot[r1],drop=FALSE] %*% t(R)
 
     # compute group-specific subspace and precisions
-    subspace <- as.list(rep(NULL, ngroups))
-    precision <- as.list(rep(NULL, ngroups))
-    for (i in seq_len(ngroups)) {
-        if (nrow(Rp[[i]]) > 0L) {
-            prec.sqrt <- backsolve(R, t(Rp[[i]][,pivot[r1],drop=FALSE]),
-                                    transpose=TRUE)
-            prec.sqrt.svd <- svd(prec.sqrt)
-
-            subspace[[i]] <- prec.sqrt.svd$u
-            precision[[i]] <- (prec.sqrt.svd$d)^2
-        } else {
-            subspace[[i]] <- matrix(0, rank, 0)
-            precision[[i]] <- numeric()
+    if(control$parallel) {
+        results <- foreach(i = seq_len(ngroups)) %dopar% {
+            if (nrow(Rp[[i]]) > 0L) {
+                prec.sqrt <- backsolve(R, t(Rp[[i]][,pivot[r1],drop=FALSE]),
+                                       transpose=TRUE)
+                prec.sqrt.svd <- svd(prec.sqrt)
+                return(list(subspace=prec.sqrt.svd$u,
+                            precision=(prec.sqrt.svd$d)^2))
+            } else {
+                return(list(subspace=matrix(0, rank, 0), precision=numeric()))
+            }
+        }
+        subspace <- lapply(results, function(x) x[['subspace']])
+        precision <- lapply(results, function(x) x[['precision']])
+        rm(results)
+    } else {
+        subspace <- as.list(rep(NULL, ngroups))
+        precision <- as.list(rep(NULL, ngroups))
+        for (i in seq_len(ngroups)) {
+            if (nrow(Rp[[i]]) > 0L) {
+                prec.sqrt <- backsolve(R, t(Rp[[i]][,pivot[r1],drop=FALSE]),
+                                       transpose=TRUE)
+                prec.sqrt.svd <- svd(prec.sqrt)
+                subspace[[i]] <- prec.sqrt.svd$u
+                precision[[i]] <- (prec.sqrt.svd$d)^2
+            } else {
+                subspace[[i]] <- matrix(0, rank, 0)
+                precision[[i]] <- numeric()
+            }
         }
     }
 
     # compute coefficient mean and covariance estimates
+    logging::loginfo("Computing mean and covariance estimates",
+                     logger="mbest.mhglm.fit")
     est0 <- NULL
     suppressWarnings({
         for (s in seq_len(control$steps)) {
-            est0 <- moment.est(coef, nfixed=rank.fixed,
-                               subspace, precision, dispersion, start.cov=est0$cov)
+            est0 <- moment.est(coef, nfixed=rank.fixed, subspace, precision,
+                               dispersion, start.cov=est0$cov, 
+                               parallel=control$parallel)
+            logging::loginfo("Refining mean and covariance estimates",
+                             logger="mbest.mhglm.fit")
         }
     })
     est <- moment.est(coef, nfixed=rank.fixed, subspace, precision, dispersion,
-                      start.cov=est0$cov)
+                      start.cov=est0$cov, parallel=control$parallel)
     mean <- est$mean
     mean.cov <- est$mean.cov
     cov <- est$cov
@@ -181,8 +230,9 @@ mhglm.fit <- function(x, z, y, group, weights = rep(1, nobs),
     dimnames(coef.cov) <- list(znames, znames)
 
     fit <- list(family = family, coefficient.mean = coef.mean,
-                coefficient.mean.cov = coef.mean.cov, coefficient.cov = coef.cov,
-                coefficients = coef, subspace = subspace, precision = precision,
+                coefficient.mean.cov = coef.mean.cov,
+                coefficient.cov = coef.cov, coefficients = coef,
+                subspace = subspace, precision = precision,
                 dispersion = dispersion.tot, df.residual = df.residual.tot,
                 R = R, rank = rank, rank.fixed = rank.fixed,
                 rank.random = rank.random, pivot = pivot, y = y, group = group,
